@@ -1,36 +1,26 @@
-# #!/usr/bin/env python
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # <sure - assertion toolbox>
 # Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
 #
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation
-# files (the "Software"), to deal in the Software without
-# restriction, including without limitation the rights to use,
-# copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following
-# conditions:
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-# OTHER DEALINGS IN THE SOFTWARE.
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import re
 import os
 import sys
 import inspect
-import platform
 import traceback
-from collections import Iterable
 
 from copy import deepcopy
 from pprint import pformat
@@ -39,11 +29,13 @@ from datetime import datetime
 try:
     from collections import Iterable
 except ImportError:
-    Iterable = (list, dict, tuple)
+    Iterable = (list, dict, tuple, set)
 
 from sure.registry import context as _registry
+from sure.magic import is_cpython, patchable_builtin
 
-version = '1.0.0alpha'
+
+version = '1.0.1alpha'
 
 
 def _get_file_name(func):
@@ -447,6 +439,19 @@ class that(object):
 
         return True
 
+    def len_is_not(self, that):
+        that = self._get_that(that)
+        length = len(self._src)
+
+        if length == that:
+            error = 'the length of %r should not be %d' % (
+                self._src,
+                that,
+            )
+            raise AssertionError(error)
+
+        return True
+
     def like(self, that):
         return self.has(that)
 
@@ -900,12 +905,46 @@ def work_in_progress(func):
     return wrapper
 
 
+def assertionmethod(func):
+    @wraps(func)
+    def wrapper(self, *args, **kw):
+        self.stack.append(func.__name__)
+        return func(self, *args, **kw)
+
+    return wrapper
+
+
+def assertionproperty(func):
+    return property(assertionmethod(func))
+
+POSITIVES = [
+    'should',
+    'does',
+    'must',
+    'when',
+]
+
+NEGATIVES = [
+    'shouldnt',
+    'dont',
+    'do_not',
+    'doesnt',
+    'does_not',
+    'doesnot',
+    'should_not',
+    'shouldnot',
+]
+
+
 class AssertionBuilder(object):
-    def __init__(self, name, negative=False):
+    def __init__(self, name=None, negative=False):
         self._name = name
         self.negative = negative
-
-        self.stack = [name]
+        self._callable_args = []
+        self._callable_kw = {}
+        self.stack = []
+        if name:
+            self.stack.append(name)
 
     def __call__(self, obj):
         self.stack.append('it')
@@ -914,21 +953,21 @@ class AssertionBuilder(object):
             obj = obj.obj
 
         self.obj = obj
-        self.__that = that(obj)
+        self._that = that(obj)
 
         return self
 
     def __getattr__(self, attr):
         special_case = False
+        special_case = attr in (POSITIVES + NEGATIVES)
 
-        special_case = attr in ('should', 'shouldnt')
-        self.negative = special_case and attr.endswith('shouldnt') or False
+        self.negative = attr in NEGATIVES
 
-        if attr == 'should':
+        if attr in POSITIVES:
             self.negative = False
             special_case = True
 
-        if attr == 'shouldnt':
+        if attr in NEGATIVES:
             self.negative = True
             special_case = True
 
@@ -939,46 +978,50 @@ class AssertionBuilder(object):
 
         return super(AssertionBuilder, self).__getattribute__(attr)
 
-    @property
-    def does(self):
-        self.negative = False
-        return self
-
-    @property
-    def dont(self):
-        self.negative = True
-        return self
-
-    doesnt = dont
-
-    @property
+    @assertionproperty
     def be(self):
-        self.stack.append('be')
         return self
 
-    @property
+    @assertionproperty
+    def when(self):
+        return self
+
+    @assertionproperty
+    def have(self):
+        return self
+
+    @assertionproperty
     def empty(self):
-        self.stack.append('empty')
         if self.negative:
-            return self.__that.len_greater_than(0)
+            return self._that.len_greater_than(0)
         else:
-            return self.__that.len_is(0)
+            return self._that.len_is(0)
 
-    @property
+    @assertionproperty
     def ok(self):
-        self.stack.append('ok')
-
         if self.negative:
             return not bool(self.obj)
         else:
             return bool(self.obj)
 
+    truthy = ok
+    true = ok
+
+    @assertionproperty
+    def falsy(self):
+        if self.negative:
+            return bool(self.obj)
+        else:
+            return not bool(self.obj)
+
+    false = falsy
+
     def __raise(self, prefix="", suffix=""):
         raise AssertionError('{0}{1}{2}'.format(
             ".".join(self.stack), self.obj, suffix))
 
+    @assertionmethod
     def within(self, *args):
-        self.stack.append('within')
         if not args:
             return self.__raise(suffix=" re")
 
@@ -993,16 +1036,18 @@ class AssertionBuilder(object):
         else:
             return collection_should.contains(self.obj)
 
+    @assertionmethod
     def equal(self, what):
-        self.stack.append('equal')
-
-        return that(self.obj).equals(what)
+        if self.negative:
+            return self._that.differs(what)
+        else:
+            return self._that.equals(what)
 
     eql = equal
     equals = equal
 
+    @assertionmethod
     def exists(self, what):
-        self.stack.append('exists')
         if self.negative:
             return not bool(what)
 
@@ -1010,22 +1055,79 @@ class AssertionBuilder(object):
 
     exist = exists
 
+    @assertionmethod
+    def length_of(self, num):
+        if self.negative:
+            return self._that.len_is_not(num)
+
+        return self._that.len_is(num)
+
+    @assertionmethod
+    def called_with(self, *args, **kw):
+        self._callable_args = args
+        self._callable_kw = kw
+        return self
+
+    @assertionmethod
+    def throw(self, *args, **kw):
+        _that = that(self.obj,
+                     with_args=self._callable_args,
+                     and_kwargs=self._callable_kw)
+
+        if self.negative:
+            msg = (u"{0} called with args {1} and kwargs {2} should "
+                   "not raise {3} but raised {4}")
+
+            exc = args and args[0] or Exception
+            try:
+                _that.raises(*args, **kw)
+                return True
+            except AssertionError, e:
+                err = msg.format(
+                    self.obj,
+                    self._that._callable_args,
+                    self._that._callable_kw,
+                    exc,
+                    e,
+                )
+                raise AssertionError(err)
+
+        return _that.raises(*args, **kw)
+
+    @assertionmethod
+    def return_value(self, value):
+        return_value = self.obj(*self._callable_args, **self._callable_kw)
+        return this(return_value).should.equal(value)
+
 this = AssertionBuilder('this')
 it = AssertionBuilder('it')
+these = AssertionBuilder('these')
+those = AssertionBuilder('those')
 
 
-if platform.python_implementation() == 'CPython':
-    from sure.magic import patchable_builtin
+if is_cpython:
+    positive_builder = AssertionBuilder(negative=False)
+    negative_builder = AssertionBuilder(negative=True)
 
-    def should(self):
-        clear_should = AssertionBuilder('should')
-        return clear_should(self)
+    def positive_assertion(name):
+        def method(self):
+            positive_builder.stack.append(name)
+            return positive_builder(self)
 
-    def shouldnt(self):
-        clear_should = AssertionBuilder('shouldnt', negative=True)
-        return clear_should(self)
+        method.__name__ = name
+        return property(method)
+
+    def negative_assertion(name):
+        def method(self):
+            negative_builder.stack.append(name)
+            return negative_builder(self)
+
+        method.__name__ = name
+        return property(method)
 
     object_handler = patchable_builtin(object)
-    object_handler['should'] = property(should)
-    object_handler['shouldnt'] = property(shouldnt)
-    object_handler['not'] = property(shouldnt)
+    for name in POSITIVES:
+        object_handler[name] = positive_assertion(name)
+
+    for name in NEGATIVES:
+        object_handler[name] = negative_assertion(name)
